/*
 * entprellen.h
 * 20111022 -us
 * 20111027 -us
 */
#ifndef _INCL_ENTPRELLEN_H
#define _INCL_ENTPRELLEN_H

#include <inttypes.h>
#include "main.h"

// --------------------------------------------------------
// Funktionsauswahl
// --------------------------------------------------------

#undef USE_KEY_RELEASE
#define USE_KEY_REPEAT
#undef USE_DEBOUNCE

// --------------------------------------------------------
// Repeat Geschwindigkeiten
// --------------------------------------------------------

#define REPEAT_MASK	(_BV( PORT_BIT(INPUT_INT1) ))
#define REPEAT_START    (550/DEBOUNCE)			// after 550ms
#define REPEAT_NEXT     (250/DEBOUNCE)			// every 250ms

#define KEY_PIN		PORT_PIN(INPUT_INT1)

// --------------------------------------------------------
// Typedef
// --------------------------------------------------------

#ifdef USE_KEY_REPEAT
#if REPEAT_START > 255U		// REPEAT_START
typedef uint16_t	rpt_cnt_t;
#else
typedef uint8_t		rpt_cnt_t;
#endif
#endif


// --------------------------------------------------------
// Externe Variable
// --------------------------------------------------------

extern volatile uint8_t key_state;
extern volatile uint8_t key_press;

#ifdef USE_KEY_RELEASE
extern volatile uint8_t key_release;
#endif

#ifdef USE_KEY_REPEAT
extern volatile uint8_t key_rpt;
#endif


// --------------------------------------------------------
// Externe (globale) Funktionen
// --------------------------------------------------------

extern uint8_t get_key_press( uint8_t key_mask );

#ifdef USE_KEY_RELEASE
extern uint8_t get_key_release( uint8_t key_mask );
#endif

#ifdef USE_KEY_REPEAT
extern uint8_t get_key_rpt( uint8_t key_mask );
extern uint8_t get_key_short( uint8_t key_mask );

# define get_key_long(key_mask)		get_key_press( get_key_rpt( key_mask ))
# define get_key_long_r(key_mask)	get_key_press( get_key_rpt( key_press & key_mask ))
# define get_key_rpt_l(key_mask)		get_key_rpt( ~key_press & key_mask )
#endif

#ifdef USE_DEBOUNCE
extern void task_key_debounce(void);
#endif


// --------------------------------------------------------
// Hilfsmacros zur Codeersparnis
// ISR_DEBOUNCE()
// --------------------------------------------------------

#ifndef USE_DEBOUNCE

#ifdef USE_KEY_RELEASE

#define ISR_DEBOUNCE()			{	\
  static uint8_t ct0 = 0xFF, ct1 = 0xFF;	\
  uint8_t i;					\
						\
  i = key_state ^ (uint8_t)(~KEY_PIN);		\
  ct0 = ~( ct0 & i );				\
  ct1 = ct0 ^ (ct1 & i);			\
  i &= ct0 & ct1;				\
  key_state ^= i;				\
  key_press |= key_state & i;			\
						\
  key_release |= (uint8_t)(~key_state) & i;	\
}

#else		// !defined( USE_KEY_RELEASE )

#define ISR_DEBOUNCE()			{	\
  static uint8_t ct0 = 0xFF, ct1 = 0xFF;	\
  uint8_t i;					\
						\
  i = key_state ^ (uint8_t)(~KEY_PIN);		\
  ct0 = ~( ct0 & i );				\
  ct1 = ct0 ^ (ct1 & i);			\
  i &= ct0 & ct1;				\
  key_state ^= i;				\
  key_press |= key_state & i;			\
}
#endif


// --------------------------------------------------------
// Hilfsmacros zur Codeersparnis
// ISR_DEBOUNCE_REPEAT()
// --------------------------------------------------------

#ifdef USE_KEY_RELEASE

#define ISR_DEBOUNCE_REPEAT()		{	\
  static uint8_t ct0 = 0xFF, ct1 = 0xFF;	\
  static rpt_cnt_t rpt;				\
  uint8_t i;					\
						\
  i = key_state ^ (uint8_t)(~KEY_PIN);		\
  ct0 = ~( ct0 & i );				\
  ct1 = ct0 ^ (ct1 & i);			\
  i &= ct0 & ct1;				\
  key_state ^= i;				\
  key_press |= key_state & i;			\
						\
  key_release |= (uint8_t)(~key_state) & i;	\
						\
  if( (key_state & REPEAT_MASK) == 0 )		\
     rpt = REPEAT_START;			\
  if( --rpt == 0 ) {				\
    rpt = REPEAT_NEXT;				\
    key_rpt |= key_state & REPEAT_MASK;		\
  }						\
}

#else		// !defined( USE_KEY_RELEASE )

#define ISR_DEBOUNCE_REPEAT()		{	\
  static uint8_t ct0 = 0xFF, ct1 = 0xFF;	\
  static rpt_cnt_t rpt;				\
  uint8_t i;					\
						\
  i = key_state ^ (uint8_t)(~KEY_PIN);		\
  ct0 = ~( ct0 & i );				\
  ct1 = ct0 ^ (ct1 & i);			\
  i &= ct0 & ct1;				\
  key_state ^= i;				\
  key_press |= key_state & i;			\
						\
  if( (key_state & REPEAT_MASK) == 0 )		\
     rpt = REPEAT_START;			\
  if( --rpt == 0 ) {				\
    rpt = REPEAT_NEXT;				\
    key_rpt |= key_state & REPEAT_MASK;		\
  }						\
}

#endif
#endif

#endif
